from typing import Optional

from coinbase.constants import API_PREFIX


def get_portfolios(self, portfolio_type: Optional[str] = None, **kwargs):
    """
    Get a list of all portfolios of a user.

    https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_getportfolios
    """
    endpoint = f"{API_PREFIX}/portfolios"

    params = {"portfolio_type": portfolio_type}

    return self.get(endpoint, params=params, **kwargs)


def create_portfolio(self, name: str, **kwargs):
    """
    Create a portfolio.

    https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_createportfolio
    """
    endpoint = f"{API_PREFIX}/portfolios"

    data = {
        "name": name,
    }

    return self.post(endpoint, data=data, **kwargs)


def get_portfolio_breakdown(self, portfolio_uuid: str, **kwargs):
    """
    Get the breakdown of a portfolio by portfolio ID.

    https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_getportfoliobreakdown
    """
    endpoint = f"{API_PREFIX}/portfolios/{portfolio_uuid}"

    return self.get(endpoint, **kwargs)


def move_portfolio_funds(
    self,
    value: str,
    currency: str,
    source_portfolio_uuid: str,
    target_portfolio_uuid: str,
    **kwargs,
):
    """
    Transfer funds between portfolios.

    https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_moveportfoliofunds
    """
    endpoint = f"{API_PREFIX}/portfolios/move_funds"

    data = {
        "funds": {
            "value": value,
            "currency": currency,
        },
        "source_portfolio_uuid": source_portfolio_uuid,
        "target_portfolio_uuid": target_portfolio_uuid,
    }

    return self.post(endpoint, data=data, **kwargs)


def edit_portfolio(self, portfolio_uuid: str, name: str, **kwargs):
    """
    Modify a portfolio by portfolio ID.

    https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_editportfolio
    """
    endpoint = f"{API_PREFIX}/portfolios/{portfolio_uuid}"

    data = {
        "name": name,
    }

    return self.put(endpoint, data=data, **kwargs)


def delete_portfolio(self, portfolio_uuid: str, **kwargs):
    """
    Delete a portfolio by portfolio ID.

    https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_deleteportfolio
    """
    endpoint = f"{API_PREFIX}/portfolios/{portfolio_uuid}"

    return self.delete(endpoint, **kwargs)



from typing import Optional

from coinbase.constants import API_PREFIX


def get_transaction_summary(
    self,
    product_type: Optional[str] = None,
    contract_expiry_type: Optional[str] = None,
    **kwargs,
):
    """
    Get a summary of transactions with fee tiers, total volume, and fees.

    https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_gettransactionsummary
    """
    endpoint = f"{API_PREFIX}/transaction_summary"

    params = {
        "product_type": product_type,
        "contract_expiry_type": contract_expiry_type,
    }

    return self.get(endpoint, params=params, **kwargs)



from typing import List

from coinbase.constants import (
    CANDLES,
    HEARTBEATS,
    LEVEL2,
    MARKET_TRADES,
    STATUS,
    TICKER,
    TICKER_BATCH,
    USER,
)


def heartbeats(self, product_ids: List[str]):
    """
    Subscribe to heartbeats channel for a list of products_ids.
    """
    self.subscribe(product_ids, [HEARTBEATS])


async def heartbeats_async(self, product_ids: List[str]):
    """
    Async subscribe to heartbeats channel for a list of products_ids.
    """
    await self.subscribe_async(product_ids, [HEARTBEATS])


def heartbeats_unsubscribe(self, product_ids: List[str]):
    """
    Unsubscribe to heartbeats channel for a list of products_ids.
    """
    self.unsubscribe(product_ids, [HEARTBEATS])


async def heartbeats_unsubscribe_async(self, product_ids: List[str]):
    """
    Async unsubscribe to heartbeats channel for a list of products_ids.
    """
    await self.unsubscribe_async(product_ids, [HEARTBEATS])


def candles(self, product_ids: List[str]):
    """
    Subscribe to candles channel for a list of products_ids.
    """
    self.subscribe(product_ids, [CANDLES])


async def candles_async(self, product_ids: List[str]):
    """
    Async subscribe to candles channel for a list of products_ids.
    """
    await self.subscribe_async(product_ids, [CANDLES])


def candles_unsubscribe(self, product_ids: List[str]):
    """
    Unsubscribe to candles channel for a list of products_ids.
    """
    self.unsubscribe(product_ids, [CANDLES])


async def candles_unsubscribe_async(self, product_ids: List[str]):
    """
    Async unsubscribe to candles channel for a list of products_ids.
    """
    await self.unsubscribe_async(product_ids, [CANDLES])


def market_trades(self, product_ids: List[str]):
    """
    Subscribe to market_trades channel for a list of products_ids.
    """
    self.subscribe(product_ids, [MARKET_TRADES])


async def market_trades_async(self, product_ids: List[str]):
    """
    Async subscribe to market_trades channel for a list of products_ids.
    """
    await self.subscribe_async(product_ids, [MARKET_TRADES])


def market_trades_unsubscribe(self, product_ids: List[str]):
    """
    Unsubscribe to market_trades channel for a list of products_ids.
    """
    self.unsubscribe(product_ids, [MARKET_TRADES])


async def market_trades_unsubscribe_async(self, product_ids: List[str]):
    """
    Async unsubscribe to market_trades channel for a list of products_ids.
    """
    await self.unsubscribe_async(product_ids, [MARKET_TRADES])


def status(self, product_ids: List[str]):
    """
    Subscribe to status channel for a list of products_ids.
    """
    self.subscribe(product_ids, [STATUS])


async def status_async(self, product_ids: List[str]):
    """
    Async subscribe to status channel for a list of products_ids.
    """
    await self.subscribe_async(product_ids, [STATUS])


def status_unsubscribe(self, product_ids: List[str]):
    """
    Unsubscribe to status channel for a list of products_ids.
    """
    self.unsubscribe(product_ids, [STATUS])


async def status_unsubscribe_async(self, product_ids: List[str]):
    """
    Async unsubscribe to status channel for a list of products_ids.
    """
    await self.unsubscribe_async(product_ids, [STATUS])


def ticker(self, product_ids: List[str]):
    """
    Subscribe to ticker channel for a list of products_ids.
    """
    self.subscribe(product_ids, [TICKER])


async def ticker_async(self, product_ids: List[str]):
    """
    Async subscribe to ticker channel for a list of products_ids.
    """
    await self.subscribe_async(product_ids, [TICKER])


def ticker_unsubscribe(self, product_ids: List[str]):
    """
    Unsubscribe to ticker channel for a list of products_ids.
    """
    self.unsubscribe(product_ids, [TICKER])


async def ticker_unsubscribe_async(self, product_ids: List[str]):
    """
    Async unsubscribe to ticker channel for a list of products_ids.
    """
    await self.unsubscribe_async(product_ids, [TICKER])


def ticker_batch(self, product_ids: List[str]):
    """
    Subscribe to ticker_batch channel for a list of products_ids.
    """
    self.subscribe(product_ids, [TICKER_BATCH])


async def ticker_batch_async(self, product_ids: List[str]):
    """
    Async subscribe to ticker_batch channel for a list of products_ids.
    """
    await self.subscribe_async(product_ids, [TICKER_BATCH])


def ticker_batch_unsubscribe(self, product_ids: List[str]):
    """
    Unsubscribe to ticker_batch channel for a list of products_ids.
    """
    self.unsubscribe(product_ids, [TICKER_BATCH])


async def ticker_batch_unsubscribe_async(self, product_ids: List[str]):
    """
    Async unsubscribe to ticker_batch channel for a list of products_ids.
    """
    await self.unsubscribe_async(product_ids, [TICKER_BATCH])


def level2(self, product_ids: List[str]):
    """
    Subscribe to level2 channel for a list of products_ids.
    """
    self.subscribe(product_ids, [LEVEL2])


async def level2_async(self, product_ids: List[str]):
    """
    Async subscribe to level2 channel for a list of products_ids.
    """
    await self.subscribe_async(product_ids, [LEVEL2])


def level2_unsubscribe(self, product_ids: List[str]):
    """
    Unsubscribe to level2 channel for a list of products_ids.
    """
    self.unsubscribe(product_ids, [LEVEL2])


async def level2_unsubscribe_async(self, product_ids: List[str]):
    """
    Async unsubscribe to level2 channel for a list of products_ids.
    """
    await self.unsubscribe_async(product_ids, [LEVEL2])


def user(self, product_ids: List[str]):
    """
    Subscribe to user channel for a list of products_ids.
    """
    self.subscribe(product_ids, [USER])


async def user_async(self, product_ids: List[str]):
    """
    Async subscribe to user channel for a list of products_ids.
    """
    await self.subscribe_async(product_ids, [USER])


def user_unsubscribe(self, product_ids: List[str]):
    """
    Unsubscribe to user channel for a list of products_ids.
    """
    self.unsubscribe(product_ids, [USER])


async def user_unsubscribe_async(self, product_ids: List[str]):
    """
    Async unsubscribe to user channel for a list of products_ids.
    """
    await self.unsubscribe_async(product_ids, [USER])



from typing import Optional

from coinbase.constants import API_PREFIX


def get_accounts(
    self, limit: Optional[int] = None, cursor: Optional[str] = None, **kwargs
):
    """
    Get a list of authenticated accounts for the current user.

    https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_getaccounts
    """
    endpoint = f"{API_PREFIX}/accounts"
    params = {"limit": limit, "cursor": cursor}

    return self.get(endpoint, params=params, **kwargs)


def get_account(self, account_uuid: str, **kwargs):
    """
    Get a list of information about an account, given an account UUID.

    https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_getaccount
    """
    endpoint = f"{API_PREFIX}/accounts/{account_uuid}"

    return self.get(endpoint, **kwargs)
